@@ -7,12 +7,11 @@ use HTTP::Server::Simple;
 use Log::Any;
 use Net::Server;
 use POSIX qw(geteuid getegid);
-use Proc::ProcessTable;
 use Server::Control::Util
   qw(get_child_pids kill_my_children is_port_active something_is_listening_msg);
 use Test::Log::Dispatch;
 use Test::Most;
-use Time::HiRes;
+use Time::HiRes qw(usleep);
 use strict;
 use warnings;
 
@@ -94,19 +93,38 @@ sub test_simple : Tests(12) {
     ok( !$ctl->is_running(), "not running" );
 }
 
-sub test_restart : Tests(8) {
+sub test_stopstart : Tests(16) {
     my $self = shift;
     my $ctl  = $self->{ctl};
     my $log  = $self->{log};
 
     ok( !$ctl->is_running(), "not running" );
-    ok( !$ctl->restart() );
-    $log->contains_ok(qr/will not attempt start/);
+    ok( $ctl->stopstart() );
+    ok( $ctl->is_running(), "is running" );
+    ok( $ctl->stop() );
+    ok( !$ctl->is_running(), "not running" );
     ok( $ctl->start() );
     ok( $ctl->is_running(), "is running" );
-    ok( $ctl->restart() );
+    ok( $ctl->stopstart() );
     ok( $ctl->is_running(), "is still running" );
     ok( $ctl->stop() );
+
+    # Make sure stopstart aborts when stop fails
+    my $orig_class  = ref($ctl);
+    my $unstoppable = Class::MOP::Class->create_anon_class(
+        superclasses => [$orig_class],
+        methods      => {
+            do_stop => sub { die "can't stop!" }
+        }
+    );
+    bless( $ctl, $unstoppable->name );
+    ok( $ctl->start() );
+    ok( !$ctl->stop() );
+    $log->contains_ok(qr/can't stop/);
+    ok( !$ctl->stopstart() );
+    $log->contains_ok(qr/could not stop.*will not attempt start/);
+    bless( $ctl, $orig_class );
+    ok( $ctl->stop() );
 }
 
 sub test_refork : Tests(7) {
@@ -120,6 +138,7 @@ sub test_refork : Tests(7) {
     my @pids = wait_for_child_pids( $proc->pid );
     ok( @pids >= 1, "at least one child pid" );
     $ctl->refork();
+    usleep(500000);    # wait for pids to die
 
     my @pids2 = wait_for_child_pids( $proc->pid );
     ok( @pids2 >= 1, "at least one child pid after refork" );
@@ -170,16 +189,17 @@ sub test_wrong_port : Tests(8) {
 
     # Tell ctl object to expect wrong port, to simulate a server not starting properly
     my $new_port = $port + 1;
-    $ctl->{port}                = $new_port;
-    $ctl->{wait_for_start_secs} = 1;
+    $ctl->{port}                 = $new_port;
+    $ctl->{wait_for_status_secs} = 1;
     ok( !$ctl->start() );
     $log->contains_ok(qr/waiting for server start/);
     $log->contains_ok(
-        qr/after .*, server .* is running \(pid .*\), but not listening to port $new_port/
+        qr/after .*, server .* appears to be running \(pid .*\), but not listening to port $new_port/
     );
     ok( $ctl->is_running(),    "running" );
     ok( !$ctl->is_listening(), "not listening" );
 
+    $ctl->{wait_for_status_secs} = 10;
     ok( $ctl->stop() );
     $log->contains_ok(qr/stopped/);
     ok( !$ctl->is_running(), "not running" );
